package ec.gpxml;

/*
Copyright 2006 by Sean Luke
Licensed under the Academic Free License version 3.0
See the file "LICENSE" for more information
 */

import ec.steadystate.*;
import ec.*;
import ec.gp.*;
import ec.gpxml.engine.Individual;
import ec.gpxml.engine.Statistics;
import ec.gpxml.engine.*;
import ec.util.*;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

import static ec.gpxml.engine.NodeType.NON_TERMINAL;
import static ec.gpxml.engine.NodeType.TERMINAL;
import static ec.gpxml.engine.RepresentationType.TREE;

/* 
 * XMLOutput.java
 * 
 * Created: Mon Jun  14 16:03:44 2010
 * By: Marhic Jerome
 */

/**
 * This class is an adaptator between ECJava and GPXML. By using this class, one may
 * save all the output (populations) of ECJava into an XML file.
 * @author Marhic Jerome
 */

public class XMLOutput implements Singleton
{
	/** log file parameter */
	public static final String P_XML_FILE = "file";

	/** The XML log */
	// not needed anymore public int xmllog;

	/* the XML file */
	File xmlFile;
	
	// the list of all the generations, that themselves contains the individuals etc etc
	List<Generation> generations;

	/** compress? */
	public static final String P_COMPRESS = "gzip";

	public static final String P_FULL = "gather-full";

	/* represent the XML output file used by the XMLGP engine */
	public Project project;

	public XMLOutput() { /*xmllog = 0; */ }

	public void setup(final EvolutionState state, final Parameter base)
	{
		project = new Project();
		project.setAttributes(1, "ECJAVA and XML GP test", "test", "today" , "SNU SC LAB", "$$$", null , null);


		xmlFile = state.parameters.getFile( base.push(P_XML_FILE),null);

		// start to generate the necessary XML code around the individuals 
		generations = new ArrayList<Generation>();

		Trial trial = new Trial();			
		trial.setAttributes(1, null, null, generations);
		
		List<Trial> trials = new ArrayList<Trial>();
		trials.add(trial);

		Treatment treatment = new Treatment();			
		treatment.setAttributes(1, "Test treatment", null, trials);
		
		List<Treatment> treatments = new ArrayList<Treatment>();
		treatments.add(treatment);

		Experiment experiment = new Experiment();			
		experiment.setAttributes(1, "14th June", "gp individual generation", null, treatments);
		
		List<Experiment> experiments = new ArrayList<Experiment>();
		experiments.add(experiment);


		project.setExperiments(experiments);
		
		/*
      if (xmlFile!=null) try
                                    {
                                    xmllog = state.output.addLog(xmlFile,Output.V_NO_GENERAL-1,false,
                                        !state.parameters.getBoolean(base.push(P_COMPRESS),null,false),
                                        state.parameters.getBoolean(base.push(P_COMPRESS),null,false));
                                    }
          catch (IOException i)
              {
              state.output.fatal("An IOException occurred while trying to create the log " + xmlFile + ":\n" + i);
              }

      state.output.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>",Output.V_NO_GENERAL, xmllog);
      state.output.println("<root> element </root>",Output.V_NO_GENERAL, xmllog);
		 */
	}




	public void postEvaluationStatistics(final EvolutionState state)
	{
		//	affiche toute la population  
		//  state.population.subpops[0].printSubpopulation(state, xmllog , Output.V_NO_GENERAL );
		Generation newgen = new Generation();
		newgen.setId( state.generation );
		generations.add(newgen);
		
		List<Individual> individus = new ArrayList<Individual>();
		
		for(int i=0 ; i< state.population.subpops[0].individuals.length ; i++ )
		{
			GPIndividual ind = (GPIndividual) state.population.subpops[0].individuals[i];
			
			Individual guy = convertECJIndToXMLGPInd(ind,i);
			if( guy != null )
				individus.add(guy);
		}
		newgen.setIndividuals(individus);
	}

	public Node convertToNode( GPNode n )
	{
		Node node = new Node();
		int nbchild = n.children.length;
		if( nbchild == 0 ) // terminus !
		{
			node.setLeafNode(TERMINAL, n.toString() );
		}
		else // not a terminal : there are children
		{
			List<Node> listchildren = new ArrayList<Node>(); // TODO decide between ArrayList, Linkedlist which is the faster ?
			for( int i=0; i< nbchild ; i++ )
			{
				Node cnode = convertToNode( n.children[i] );
				listchildren.add(cnode);
			}
			node.setNonTerminalNode(NON_TERMINAL, n.toString(), listchildren);
		}
		return node;
	}
	
	/* convertir la representation ECJava d'un individu en une representation GPXML */
	public Individual convertECJIndToXMLGPInd(GPIndividual ind , int id)
	{
		Individual guy = new Individual();
	/*	guy.setId(id);
		float fitness = 100 * ind.fitness.fitness();
		if( fitness != 100)
			return null;
		
		guy.setFitness( (int) fitness );
	*/	
		GPNode root = ind.trees[0].child; // we take only the first tree
		Node node = convertToNode(root);		
		
		Representation representation = new Representation();			
		representation.setAttributes(1, TREE, null, node);
		
		List<Representation> representations = new ArrayList<Representation>();
		representations.add(representation);
		guy.setRepresentations(representations);

		return guy;
	}

	/** Logs the best individual of the run. */
	public void finalStatistics(final EvolutionState state, final int result)
	{

		if(xmlFile != null )
		{
			XMLGP.serializeToFile(project, xmlFile.getAbsolutePath() );
		}
	} 

}
