package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Experiment.<br/>
 * It has setters and getters to all the attributes of Experiment.
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize experiment XML element to experiment object and 
 * to serialize experiment object to experiment XML element.
 * 
 * @author dharani
 *
 */

@XmlDataType(defaultElementName = "experiment")
public class Experiment {
   
	/** Id of the experiment */ 
	@XmlAttribute()
    private int id;
	
	/** Date on which the experiment is done */
	@XmlAttribute()
    private String date;
	
	/** Description of the experiment */
	@XmlAttribute()
    private String description;
	
	/** List of statistical measures of the experiment */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	private List<Statistics> statistics;
	
	/** List of treatments contained in the experiment */
	@XmlElementList(items = {@ListItem(name = "treatment")})
	private List<Treatment> treatments;
	

	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 * 
	 * @param id the id of the experiment
	 * @param date the date to set
	 * @param description the description of the experiment to set
	 * @param statistics the list of statistics to set
	 * @param treatments the list of treatments to set
	 */
	public void setAttributes(int id, String date, String description, List<Statistics> statistics, List<Treatment> treatments) {
		this.id = id;
		this.date = date;
		this.description = description;
		this.statistics = statistics;
		this.treatments = treatments;
	}	
	
	/**
	 * Retrieves the id of the experiment.
	 * @return the id of the experiment
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the experiment.
	 * @param id the id of the experiment
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Retrieves the date on which the experiment is done.
	 * @return the date on which the experiment is done
	 */	
	public String getDate() {
		return date;
	}

	/**
	 * Sets the date on which the experiment is done.
	 * @param date the date to set
	 */
	public void setDate(String date) {
		this.date = date;
	}

	/**
	 * Retrieves the description of the experiment.
	 * @return the description of the experiment
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the description of the experiment.
	 * @param description the description of the experiment to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Retrieves the list of statistical measures of the experiment.
	 * @return the list of statistical measures of the experiment
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of the experiment.
	 * @param statistics the list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Retrieves the list of treatments contained in the experiment.
	 * @return the list of treatments in the experiment
	 */
	public List<Treatment> getTreatments() {
		return treatments;
	}

	/**
	 * Sets the list of treatments contained in the experiment.
	 * @param treatments the list of treatments to set
	 */
	public void setTreatments(List<Treatment> treatments) {
		this.treatments = treatments;
	}
}