package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Generation. <br/>
 * It has setters and getters to all the attributes of Generation. 
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize generation XML element to generation object and 
 * to serialize generation object to generation XML element.
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "generation")
public class Generation {
   
	/** Id of the generation */
	@XmlAttribute()
    private  int id;

	/** List of statistical measures of the generation */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	private  List<Statistics> statistics;
	
	/** List of individuals of the generation */
	@XmlElementList(items = {@ListItem(name = "individual")})
	private  List<Individual> individuals;
	
	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 * 
	 * @param id the id of the generation
	 * @param statistics the list of statistics to set
	 * @param individuals the list of individuals to set
	 */
	public void setAttributes(int id, List<Statistics> statistics, List<Individual> individuals)
	{
		this.id = id;
		this.statistics = statistics;
		this.individuals = individuals;
	}

	/**
	 * Returns the id of the generation.
	 * @return the id of the generation
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the generation.
	 * @param id the id of the generation to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the list of statistical measures of the generation.
	 * @return the list of statistical measures of the generation
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of the generation.
	 * @param statistics the list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the list of individuals of the generation.
	 * @return the list of individuals of the generation.
	 */
	public List<Individual> getIndividuals() {
		return individuals;
	}

	/**
	 * Sets the list of individuals of the generation.
	 * @param individuals the list of individuals to set
	 */
	public void setIndividuals(List<Individual> individuals) {
		this.individuals = individuals;
	}	
}
