package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Individual. <br/>
 * It has setters and getters to all the attributes of Individual.
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize individual XML element to individual object and 
 * to serialize individual object to individual XML element.
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "individual")
public class Individual {
   
	/** Id of the individual */
	@XmlAttribute()
    int id;
	
	/** fitness of this individual with regard to the 
	 * 	considered problem
	 */
//	@XmlAttribute()
//	int fitness;
	
	/** List of statistical measures of the individual */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	List<Statistics> statistics;
	
	/** List of representations of the individual */
    @XmlElementList(items = {@ListItem(name = "representation")})
	List<Representation> representations;
    
    /**
     * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 *  
     * @param id the id of the individual to set
     * @param statistics the list of statistics to set
     * @param representations the list of representations to set
     */
    public void setAttributes(int id, List<Statistics> statistics, List<Representation> representations)
    {
    	this.id = id;
    //	this.fitness = f;
    	this.statistics = statistics;
		this.representations = representations;
    }

	/**
	 * Returns the id of the individual.
	 * @return the id of the individual
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the individual. 
	 * @param id the id of the individual to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the fitness of the individual.
	 * @return the fitness of the individual
	 */
/*	public int getFitness() {
		return fitness;
	}*/

	/**
	 * Sets the fitness of the individual. 
	 * @param f the fitness of the individual to set
	 */
/*	public void setFitness(int f) {
		this.fitness = f;
	}*/
	/**
	 * Returns the list of statistical measures of the individual.
	 * @return the list of statistical measures of the individual.
	 */
	public List<Statistics> getStatistics() {
		return statistics;		
	}

	/**
	 * Sets the list of statistical measures of the individual.
	 * @param statistics the list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the list of representations of the individual.
	 * @return the list of representations of the individual
	 */
	public List<Representation> getRepresentations() {
		return representations;
	}

	/**
	 * Sets the list of representations of this individual.
	 * @param representations the list of representations to set
	 */
	public void setRepresentations(List<Representation> representations) {
		this.representations = representations;
	} 
}