package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Node. <br/>
 * It has setters and getters to all the attributes of Node. 
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize node XML element to node object and 
 * to serialize node object to node XML element.
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "node")
public class Node {
    
	//@XmlAttribute()
    //int id;
	
	/** The type of the node */	 
	@XmlAttribute()
    private NodeType type;

	/** The value of the node */
	@XmlAttribute()
    private String value;	
	
	/** List of children of the node */
	@XmlElementList(items = {@ListItem(name = "node")})
	private List<Node> children;

	/**
	 * Sets non terminal node and sets all attributes at once.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * 
	 * @param type the type of the node to set
	 * @param value the value of the node to set
	 * @param children the list of children to set
	 */
	public void setNonTerminalNode(NodeType type, String value, List<Node> children)
	{
		this.type = type;
		this.value = value;
		this.children = children;
	}
	
	/**
	 * Sets terminal node and sets all attributes at once.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * 
	 * @param type the type of the node to set
	 * @param value the value of the node to set
	 */
	public void setLeafNode(NodeType type, String value)
	{
		this.type = type;
		this.value = value;
	}
	
	/**
	 * Returns the type of the node.
	 * @return the type of the node
	 */
	public NodeType getType() {
		return type;
	}

	/**
	 * Sets the type of the node.
	 * @param type the type of the node to set
	 */
	public void setType(NodeType type) {
		this.type = type;
	}

	/**
	 * Returns the value of the node.
	 * @return the value of the node
	 */
	public String getValue() {
		return value;
	}

	/**
	 * Sets the value of the node.
	 * @param value the value of the node to set
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 * Returns the list of children of the node. 
	 * @return the list of children of the node 
	 */
	public List<Node> getChildren() {
		return children;
	}

	/**
	 * Sets the list of children of the node. 
	 * @param children the list of children to set
	 */
	public void setChildren(List<Node> children) {
		this.children = children;
	}
}