package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the root data holder for project.<br/>
 * It has setters and getters to all the attributes of Project.
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize project XML element to project object and 
 * to serialize project object to project XML element.
 *  
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "project")
public class Project {
   
	/** Id of the project */
	@XmlAttribute()
    private int id;
	
	/** Name of the project */
	@XmlAttribute()
    private String name;
	
	/** Description of the project */
	@XmlAttribute()
    private String description;
	
	/** Date on which the project started */
	@XmlAttribute()
    private String date;
	
	/** Organization which handles the project */
	@XmlAttribute()
    private String organization;
	
	/** project fund */
	@XmlAttribute()
    private String fund;
	
	/** List of statistical measures of the project */
	@XmlElementList(items = {@ListItem(name = "stats")})
	private List<Statistics> statistics;
	
	/** List of experiments for this project */
	@XmlElementList(items = {@ListItem(name = "experiment")})
	private List<Experiment> experiments;
	
	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 *  
	 * @param id the project id to set
	 * @param name the name of project to set
	 * @param description the description of the project to set
	 * @param date the date of the project to set
	 * @param organization the name of the organization which handles the project to set
	 * @param fund the project fund to set
	 * @param statistics the project fund to set
	 * @param experiments the list of experiments of the project to set
	 */
	public void setAttributes(int id, String name, String description, String date, String organization, String fund, List<Statistics> statistics, List<Experiment> experiments)
	{
		this.id = id;
		this.name = name;
		this.description = description;
		this.date = date;
		this.organization = organization;
		this.fund = fund;
		this.statistics = statistics;
		this.experiments = experiments;
	}

	/**
	 * Returns the id of the project.
	 * @return the project id
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the project.
	 * @param id the project id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the name of the project.
	 * @return the name of the project
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the project.
	 * @param name the name of project to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the description of the project.
	 * @return the description of the project
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the description of the project.
	 * @param description the description of the project to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Returns the date of the project.
	 * @return the date of the project
	 */
	public String getDate() {
		return date;
	}

	/**
	 * Sets the date of the project.
	 * @param date the date of the project to set
	 */
	public void setDate(String date) {
		this.date = date;
	}

	/**
	 * Returns the name of the organization which handles the project.
	 * @return the name of the organization which handles the project 
	 */
	public String getOrganization() {
		return organization;
	}

	/**
	 * Sets the name of the organization which handles the project.
	 * @param organization the name of the organization which handles the project to set
	 */
	public void setOrganization(String organization) {
		this.organization = organization;
	}

	/**
	 * Returns the project fund.
	 * @return the project fund
	 */
	public String getFund() {
		return fund;
	}

	/**
	 * Sets the project fund.
	 * @param fund the project fund to set
	 */
	public void setFund(String fund) {
		this.fund = fund;
	}

	/**
	 * Return the list of statistical measures of the project.
	 * @return the statistics of the project
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of the project.
	 * @param statistics the statistics of the project to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the list of experiments of the project.
	 * @return the list of experiments of the project
	 */
	public List<Experiment> getExperiments() {
		return experiments;
	}

	/**
	 * Sets the list of experiments of the project.
	 * @param experiments the list of experiments of the project to set
	 */
	public void setExperiments(List<Experiment> experiments) {
		this.experiments = experiments;
	}
}