package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElement;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Representation. <br/>
 * It has setters and getters to the attributes of Representation. 
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize representation XML element to representation object and 
 * to serialize representation object to representation XML element.
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "representation")
public class Representation {
    
	/** Id of the representation */
	@XmlAttribute()
    private int id;
	
	/** Type of the representation */
	@XmlAttribute()
    private RepresentationType type;
	
	/** List of statistical measures of the experiment */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	private List<Statistics> statistics;
	
	/** Node which represents the results*/
	@XmlElement()
    private Node node;
	
	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 * 
	 * @param id the id of representation to set
	 * @param type the type of representation to set
	 * @param statistics the list of statistics to set
	 * @param node the node which represents the results to set
	 */
	public void setAttributes(int id, RepresentationType type, List<Statistics> statistics, Node node)
	{
		this.id = id;
		this.type = type;
		this.statistics = statistics;
		this.node = node;
	}

	/**
	 * Returns the id of the representation.
	 * @return the id of the representation
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the representation.
	 * @param id the id of representation to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the type of the representation.
	 * @return the type of the representation
	 */
	public RepresentationType getType() {
		return type;
	}

	/**
	 * Sets the type of the representation.
	 * @param type the type of representation to set
	 */
	public void setType(RepresentationType type) {
		this.type = type;
	}

	/**
	 * Returns the list of statistical measures of the representation.
	 * @return the list of statistical measures of the representation 
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of the representation.
	 * @param statistics list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the node which represents the results.
	 * @return the node which represents the results
	 */
	public Node getNode() {
		return node;
	}

	/**
	 * Sets the node which represents the results.
	 * @param node the node which represents the results to set
	 */
	public void setNode(Node node) {
		this.node = node;
	}	
}
