package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Treatment. <br/>
 * It has setters and getters to the attributes of Treatment.  
 *  
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize treatment XML element to treatment object and 
 * to serialize treatment object to treatment XML element. 
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "treatment")
public class Treatment {
   
	/** ID of the treatment */
	@XmlAttribute()
    private int id;

	/** Description of the treatment */
	@XmlAttribute()
    private String description;
	
	/**List of statistical measures of treatment */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	private List<Statistics> statistics;
	
	/** List of trials of treatment */
	@XmlElementList(items = {@ListItem(name = "trial")})
	private List<Trial> trials;
	
	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 * 
	 * @param id the id to set
	 * @param description the description of the treatment to set
	 * @param statistics the list of statistics to set
	 * @param trials the list of trials to set
	 */
	public void setAttributes(int id, String description, List<Statistics> statistics, List<Trial> trials)
	{
		this.id = id;
		this.description = description;
		this.statistics = statistics;
		this.trials = trials;
	}

	/**
	 * Returns the id of the treatment.
	 * @return the id of the treatment.
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the treatment.
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the description of the treatment.
	 * @return the description of the treatment
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the description of the treatment.
	 * @param description the description of the treatment to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Returns the list of statistical measures of treatment.
	 * @return the list of statistical measures of treatment.
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of treatment.
	 * @param statistics the list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the list of trials of the treatment.
	 * @return the list of trials of the treatment
	 */
	public List<Trial> getTrials() {
		return trials;
	}

	/**
	 * Sets the list of trials of the treatment.
	 * @param trials the list of trials to set
	 */
	public void setTrials(List<Trial> trials) {
		this.trials = trials;
	}	
}