package ec.gpxml.engine;

import java.util.List;

import org.jsefa.xml.annotation.ListItem;
import org.jsefa.xml.annotation.XmlAttribute;
import org.jsefa.xml.annotation.XmlDataType;
import org.jsefa.xml.annotation.XmlElementList;

/**
 * This class is the data holder for Trial. <br/>
 * It has setters and getters to the members of Trial.
 * 
 * <p>It also has the annotations which will be used by <b>jsefa</b>
 * to deserialize trial XML element to trial object and 
 * to serialize trial object to trial XML element. 
 * 
 * @author dharani
 *
 */
@XmlDataType(defaultElementName = "trail")
public class Trial {
   
	/** Id of the trial */
	@XmlAttribute()
    private int id;
	
	/**Attribute of trial */
	@XmlAttribute()
    private String attribute;
	
	/** List of statistical measures of the trial */
	@XmlElementList(items = {@ListItem(name = "statistics")})
	private List<Statistics> statistics;
	
	/** List of generations of the trial */
	@XmlElementList(items = {@ListItem(name = "generation")})
	private List<Generation> generations;
	
	/**
	 * Sets all the attributes of this class.
	 * This method is just for convenience to set all attributes at once instead of setting one by one.
	 * Null is passed if an attribute doesn't need to be set.
	 *  
	 * @param id the id of the trial to set
	 * @param attribute the attribute to set
	 * @param statistics the list of statistics to set
	 * @param generations the list of generations to set
	 */
	public void setAttributes(int id, String attribute, List<Statistics> statistics, List<Generation> generations)
	{
		this.id = id;
		this.attribute = attribute;
		this.statistics = statistics;
		this.generations = generations;		
	}

	/**
	 * Returns the id of the trial.
	 * @return the id of the trial
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id of the trial.
	 * @param id the id of the trial to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * Returns the attribute of the trial.
	 * @return the attribute of the trial
	 */
	public String getAttribute() {
		return attribute;
	}

	/**
	 * Sets the attribute of the trial.
	 * @param attribute the attribute to set
	 */
	public void setAttribute(String attribute) {
		this.attribute = attribute;
	}

	/**
	 * Returns the list of statistical measures of the trial.
	 * @return the list of statistical measures of the trial.
	 */
	public List<Statistics> getStatistics() {
		return statistics;
	}

	/**
	 * Sets the list of statistical measures of the trial.
	 * @param statistics the list of statistical measures to set
	 */
	public void setStatistics(List<Statistics> statistics) {
		this.statistics = statistics;
	}

	/**
	 * Returns the list of generations of the trial.
	 * @return the list of generations of the trial.
	 */
	public List<Generation> getGenerations() {
		return generations;
	}

	/**
	 * Sets the list of generations of the trial..
	 * @param generations the list of generations to set
	 */
	public void setGenerations(List<Generation> generations) {
		this.generations = generations;
	}
}